<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - @year@ RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\export;

defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;


/**
 * Exports HikaShop products.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class Product extends \CsviModelExports
{
	/**
	 * The domain name for URLs.
	 *
	 * @var    string
	 * @since  6.0
	 */
	private $domainName;

	/**
	 * The HikaShop helper
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop
	 * @since  6.5.0
	 */
	protected $helper;

	/**
	 * The HikaShop config helper
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop_Config
	 * @since  6.0.0
	 */
	protected $helperConfig;

	/**
	 * Array of prices per product
	 *
	 * @var    array
	 * @since  6.0
	 */
	private $prices = array();

	/**
	 * List of custom fields
	 *
	 * @var    array
	 * @since  6.0
	 */
	private $customFields = array();

	/**
	 * List of characteristic fields
	 *
	 * @var    array
	 * @since  6.6.0
	 */
	private $characteristicFields = array();

	/**
	 * List of parent codes
	 *
	 * @var    array
	 * @since  6.6.0
	 */
	private $parentIds = array();

	/**
	 * Export the data.
	 *
	 * @return  bool  True if body is exported | False if body is not exported.
	 *
	 * @since   6.0
	 *
	 * @throws  \Exception
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Get some basic data
			$this->domainName = $this->settings->get('hostname');
			require_once JPATH_PLUGINS . '/csviaddon/hikashop/com_hikashop/helper/com_hikashop_category_product.php';
			$categoryHelper = new \Com_HikashopHelperCom_Hikashop_Category_Product($this->db, $this->template, $this->log);
			$this->loadCustomFields();
			$this->loadCharacteristicFields();
			$exportFields = $this->fields->getFields();

			// Group by fields
			$groupFields   = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupBy       = [];
			$groupByFields = [];

			if (isset($groupFields->name))
			{
				$groupByFields = array_flip($groupFields->name);
			}

			// Sort selected fields
			$sortFields   = json_decode($this->template->get('sortfields', '', 'string'));
			$sortBy       = [];
			$sortByFields = [];

			if (isset($sortFields->name))
			{
				foreach ($sortFields->sortby as $key => $sortName)
				{
					$sortByFields[$sortFields->name[$key]] = $sortName;
				}
			}

			// Build something fancy to only get the fieldnames the user wants
			$userFields = [];

			foreach ($exportFields as $field)
			{
				$sortDirection = ($sortByFields[$field->field_name]) ?? 'ASC';

				switch ($field->field_name)
				{
					case 'brand':
						$userFields[] = $this->db->quoteName('p.product_manufacturer_id');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('p.product_manufacturer_id');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('p.product_manufacturer_id') . ' ' . $sortDirection;
						}
						break;
					case 'product_options':
					case 'related_products':
					case 'category_path':
					case 'characteristics':
					case 'picture_url':
					case 'picture_url_thumb':
					case 'file_path':
					case 'file_name':
					case 'file_description':
					case 'file_ordering':
						$userFields[] = $this->db->quoteName('p.product_id');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('p.product_id');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('p.product_id') . ' ' . $sortDirection;
						}
						break;
					case 'product_tax':
					case 'product_tax_category':
						$userFields[] = $this->db->quoteName('p.product_tax_id');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('p.product_tax_id');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('p.product_tax_id') . ' ' . $sortDirection;
						}
						break;
					case 'product_url':
						$userFields[] = $this->db->quoteName('p.product_id');
						$userFields[] = $this->db->quoteName('p.product_alias');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('p.product_id');
							$groupBy[] = $this->db->quoteName('p.product_alias');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('p.product_id') . ' ' . $sortDirection;
							$sortBy[] = $this->db->quoteName('p.product_alias') . ' ' . $sortDirection;
						}
						break;
					case 'product_price':
						$userFields[] = $this->db->quoteName('pr.price_value');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('pr.price_value');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('pr.price_value') . ' ' . $sortDirection;
						}
						break;
					case 'product_parent_code':
						$userFields[] = $this->db->quoteName('p.product_parent_id');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('p.product_parent_id');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('p.product_parent_id') . ' ' . $sortDirection;
						}
						break;
					case 'custom':
						break;
					default:

						// Do not include custom fields into the query
						if (!array_key_exists($field->field_name, $this->characteristicFields))
						{
							$userFields[] = $this->db->quoteName('p.' . $field->field_name);

							if (array_key_exists($field->field_name, $groupByFields))
							{
								$groupBy[] = $this->db->quoteName('p.' . $field->field_name);
							}

							if (array_key_exists($field->field_name, $sortByFields))
							{
								$sortBy[] = $this->db->quoteName('p.' . $field->field_name) . ' ' . $sortDirection;
							}
						}
						break;
				}
			}

			/**
			 * Export SQL Query
			 * Get all products - including items
			 * as well as products without a price
			 */
			$userFields = array_unique($userFields);
			$query = $this->db->getQuery(true)
				->select(implode(",\n", $userFields))
				->from($this->db->quoteName('#__hikashop_product', 'p'));

			// If variant product has to be exported
			if ($this->template->get('child_only', false))
			{
				$query->where($this->db->quoteName('p.product_type') . ' = ' . $this->db->quote('variant'), 'OR');
			}

			if ($this->template->get('parent_only', false))
			{
				$query->where($this->db->quoteName('p.product_type') . ' = ' . $this->db->quote('main'));
			}

			$query->leftJoin(
					$this->db->quoteName('#__hikashop_price', 'pr')
					. ' ON ' . $this->db->quoteName('p.product_id') . ' = ' . $this->db->quoteName('pr.price_product_id')
				);

			// Filter by published state
			$product_publish = $this->template->get('publish_state');

			if ($product_publish !== '' && ($product_publish === '1' || $product_publish === '0'))
			{
				$query->where('p.product_published = ' . (int) $product_publish);
			}

			// Filter by category
			$productCategories = $this->template->get('product_categories', false);

			// Sanity check
			ArrayHelper::toInteger($productCategories);

			if ($productCategories && $productCategories[0] !== '')
			{
				$query->leftJoin(
					$this->db->quoteName('#__hikashop_product_category', 'c')
					. ' ON ' . $this->db->quoteName('c.product_id') . ' = ' . $this->db->quoteName('p.product_id')
				);

				$query->where($this->db->quoteName('c.category_id') . ' IN (' . implode(',', $productCategories) . ')');
			}

			// Filter by product SKU
			$productskufilter = $this->template->get('productskufilter');

			if ($productskufilter)
			{
				$productskufilter .= ',';

				if (strpos($productskufilter, ','))
				{
					$skus = explode(',', $productskufilter);
					$wildcard = '';
					$normal = array();

					foreach ($skus as $sku)
					{
						if (!empty($sku))
						{
							if (is_int(strpos($sku, '%')))
							{
								$wildcard .= $this->db->quoteName('p.product_code') . ' LIKE ' . $this->db->quote($sku) . ' OR ';
							}
							else
							{
								$normal[] = $this->db->quote($sku);
							}
						}
					}

					if (substr($wildcard, -3) === 'OR ')
					{
						$wildcard = substr($wildcard, 0, -4);
					}

					if (!empty($wildcard) && !empty($normal))
					{
						$query->where('(' . $wildcard . ' OR ' . $this->db->quoteName('p.product_code') . ' IN (' . implode(',', $normal) . '))');
					}
					elseif (!empty($wildcard))
					{
						$query->where('(' . $wildcard . ')');
					}
					elseif (!empty($normal))
					{
						$query->where('(' . $this->db->quoteName('p.product_code') . ' IN (' . implode(',', $normal) . '))');
					}
				}
			}

			// Filter by stocklevel start
			$stocklevelstart = $this->template->get('stocklevelstart', 'none', 'cmd');

			if ($stocklevelstart !== 'none')
			{
				$stocklevelstart = (int) $stocklevelstart;
				$query->where($this->db->quoteName('p.product_quantity') . ' >= ' . $stocklevelstart);
			}

			// Filter by stocklevel end
			$stocklevelend = $this->template->get('stocklevelend', 'none', 'cmd');

			if ($stocklevelend !== 'none' || ($stocklevelend === 0 && $stocklevelstart === 0 ))
			{
				$stocklevelend = (int) $stocklevelend;
				$query->where($this->db->quoteName('p.product_quantity') . ' <= ' . $stocklevelend);
			}

			// Filter by shopper group id
			$shopper_group = $this->template->get('shopper_groups', array());

			if ($shopper_group && $shopper_group[0] !== 'none')
			{
				$where = array();

				foreach ($shopper_group as $group)
				{
					$where[] = $this->db->quoteName('p.product_access') . ' LIKE ' . $this->db->quote('%,' . $group . ',%');
				}

				if (!empty($where))
				{
					$query->where('(' . implode(' OR ', $where) . ')');
				}
			}

			// Filter by manufacturer
			$manufacturer = $this->template->get('manufacturers', array());

			if ($manufacturer && !empty($manufacturer) && $manufacturer[0] !== 'none')
			{
				$query->where($this->db->quoteName('p.product_manufacturer_id') . " IN ('" . implode("','", $manufacturer) . "')");
			}

			// Filter on price shopper group
			$shopper_group_price = $this->template->get('shopper_group_price', array());

			if ($shopper_group_price)
			{
				$where = array();

				foreach ($shopper_group_price as $group)
				{
					$where[] = $this->db->quoteName('pr.price_access') . ' LIKE ' . $this->db->quote('%,' . $group . ',%');
				}

				if (!empty($where))
				{
					$query->where('(' . implode(' OR ', $where) . ')');
				}
			}

			// Filter on price from
			$priceoperator = $this->template->get('priceoperator', 'gt');
			$pricefrom = $this->template->get('pricefrom', false);
			$priceto = $this->template->get('priceto', 0, 'float');

			if ($pricefrom !== false)
			{
				switch ($priceoperator)
				{
					case 'gt':
						$query->where(
							'ROUND('
							. $this->db->quoteName('pr.price_value') . ', '
							. $this->template->get('export_price_format_decimal', 2, 'int') . ') > ' . $pricefrom
						);
						break;
					case 'eq':
						$query->where(
							'ROUND('
							. $this->db->quoteName('pr.price_value') . ', '
							. $this->template->get('export_price_format_decimal', 2, 'int') . ') = ' . $pricefrom
						);
						break;
					case 'lt':
						$query->where(
							'ROUND('
							. $this->db->quoteName('pr.price_value') . ', '
							. $this->template->get('export_price_format_decimal', 2, 'int') . ') < ' . $pricefrom
						);
						break;
					case 'bt':
						$query->where(
							'ROUND('
							. $this->db->quoteName('pr.price_value') . ', '
							. $this->template->get('export_price_format_decimal', 2, 'int') . ') BETWEEN ' . $pricefrom . ' AND ' . $priceto
						);
						break;
				}
			}

			// Filter on product quantities
			$price_quantity_start = $this->template->get('price_quantity_start', false);
			$price_quantity_end = $this->template->get('price_quantity_end', false);

			if ($price_quantity_start && $price_quantity_start >= 0)
			{
				if ($price_quantity_end && $price_quantity_end >= 0)
				{
					$query->where($this->db->quoteName('pr.price_min_quantity') . ' BETWEEN ' . (int) $price_quantity_start . ' AND ' . (int) $price_quantity_end);
				}
				else
				{
					$query->where($this->db->quoteName('pr.price_min_quantity') . ' = ' . (int) $price_quantity_start);
				}
			}

			// Group the fields
			$groupBy = array_unique($groupBy);

			if (!empty($groupBy))
			{
				$query->group($groupBy);
			}

			// Sort set fields
			$sortBy = array_unique($sortBy);

			if (!empty($sortBy))
			{
				$query->order($sortBy);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logcount = $this->db->getNumRows();

			if ($logcount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					// Reset the prices
					$this->prices = array();

					// Process all the export fields
					foreach ($exportFields as $field)
					{
						// Get the field name
						$fieldName = $field->field_name;

						// Set the field value
						if (isset($record->$fieldName))
						{
							$fieldValue = $record->$fieldName;
						}
						else
						{
							$fieldValue = '';
						}

						// Process the field
						switch ($fieldName)
						{
							case 'category_path':
								$fieldValue = $categoryHelper->createCategoryPath($record->product_id);
								break;
							case 'related_products':
								$related_records = $this->getProductRelations($record->product_id, 'related');

								if (is_array($related_records))
								{
									$fieldValue = implode('|', $related_records);
								}
								else
								{
									$fieldValue = '';
								}
								break;
							case 'product_options':
								$option_records = $this->getProductRelations($record->product_id, 'options');

								if (is_array($option_records))
								{
									$fieldValue = implode('|', $option_records);
								}
								else
								{
									$fieldValue = '';
								}
								break;
							case 'characteristics':
								$characteristics = $this->getCharacteristics($record->product_id);

								if (is_array($characteristics))
								{
									$fieldValue = implode('|', $characteristics);
								}
								else
								{
									$fieldValue = '';
								}
								break;
							case 'product_price':
								$fieldValue = number_format(
									$record->price_value,
									$this->template->get('export_price_format_decimal', 2, 'int'),
									$this->template->get('export_price_format_decsep'),
									$this->template->get('export_price_format_thousep')
								);

								if (strlen(trim($fieldValue)) == 0)
								{
									$fieldValue = $field->default_value;
								}

								if ($this->template->get('add_currency_to_price'))
								{
									$fieldValue = $record->currency_code_3 . ' ' . $fieldValue;
								}
								break;
							case 'product_url':
								// Let's create a SEF URL
								$url = 'option=com_hikashop&ctrl=product&task=show&' .
									'cid=' . $record->product_id .
									'&name=' . $record->product_alias .
									'&Itemid=' . $this->template->get('itemid', 1, 'int');
								$fieldValue = $this->sef->getSefUrl('index.php?' . $url);

								// Add the suffix
								if (!empty($fieldValue))
								{
									$fieldValue .= $this->template->get('producturl_suffix');
								}
								break;
							case 'picture_url':
							case 'picture_url_thumb':
								if (substr($this->domainName, -1) !== '/' && substr($this->helperConfig->get('uploadfolder'), 0, 1) !== '/')
								{
									$this->domainName = $this->domainName . '/';
								}

								$path = $this->domainName . $this->helperConfig->get('uploadfolder');

								if ($fieldName === 'picture_url_thumb')
								{
									$path .= 'thumbnails/100x100f/';
								}

								$query = $this->db->getQuery(true)
									->select('CONCAT(' . $this->db->quote($path) . ', ' . $this->db->quoteName('file_path') . ')')
									->from($this->db->quoteName('#__hikashop_file'))
									->where($this->db->quoteName('file_ref_id') . ' = ' . (int) $record->product_id)
									->where($this->db->quoteName('file_type') . ' = ' . $this->db->quote('product'))
									->order($this->db->quoteName('file_ordering'));
								$this->db->setQuery($query, 0, $this->template->get('picture_limit', 1));
								$images = $this->db->loadColumn();

								$fieldValue = implode(',', $images);
								break;
							case 'file_path':
							case 'file_name':
							case 'file_description':
							case 'file_ordering':
								$imageArray = array();
								$fieldValue = '';
								$query      = $this->db->getQuery(true)
									->select($this->db->quoteName($fieldName))
									->from($this->db->quoteName('#__hikashop_file'))
									->where($this->db->quoteName('file_ref_id') . ' = ' . (int) $record->product_id)
									->where($this->db->quoteName('file_type') . ' = ' . $this->db->quote('product'))
									->order($this->db->quoteName('file_ordering'));
								$this->db->setQuery($query);
								$images = $this->db->loadObjectList();

								foreach ($images as $image)
								{
									$imageArray[] = $image->$fieldName;
								}

								if ($imageArray)
								{
									$fieldValue = implode('|', $imageArray);
								}
								break;
							case 'product_tax':
								$query = $this->db->getQuery(true)
									->select($this->db->quoteName('t.tax_rate'))
									->from($this->db->quoteName('#__hikashop_tax', 't'))
									->leftJoin(
										$this->db->quoteName('#__hikashop_taxation', 'tx')
										. ' ON ' . $this->db->quoteName('tx.tax_namekey') . ' = ' . $this->db->quoteName('t.tax_namekey')
									)
									->leftJoin(
										$this->db->quoteName('#__hikashop_category', 'c')
										. ' ON ' . $this->db->quoteName('c.category_namekey') . ' = ' . $this->db->quoteName('tx.category_namekey')
									)
									->where($this->db->quoteName('c.category_id') . ' = ' . (int) $record->product_tax_id);
								$this->db->setQuery($query);

								$fieldValue = ($this->db->loadResult() * 100);
								break;
							case 'product_tax_category':
								$query = $this->db->getQuery(true)
									->select($this->db->quoteName('category_name'))
									->from($this->db->quoteName('#__hikashop_category'))
									->where($this->db->quoteName('category_type') . ' = ' . $this->db->quote('tax'))
									->where($this->db->quoteName('category_id') . ' = ' . (int) $record->product_tax_id);
								$this->db->setQuery($query);

								$fieldValue = $this->db->loadResult();
								break;
							case 'brand':
								$query = $this->db->getQuery(true)
									->select($this->db->quoteName('category_name'))
									->from($this->db->quoteName('#__hikashop_category'))
									->where($this->db->quoteName('category_type') . ' = ' . $this->db->quote('manufacturer'))
									->where($this->db->quoteName('category_id') . ' = ' . (int) $record->product_manufacturer_id);
								$this->db->setQuery($query);

								$fieldValue = $this->db->loadResult();
								break;
							case 'product_access':
								if ($fieldValue !== 'all')
								{
									$accessIds   = explode(',', $fieldValue);
									$accessIds   = array_unique($accessIds);
									$accessIds   = ArrayHelper::toInteger($accessIds);
									$accessNames = array();

									foreach ($accessIds as $index => $accessId)
									{
										if ($accessId)
										{
											$accessNames[] = $this->helper->getAccessLevelName($accessId);
										}
									}

									$fieldValue = implode('|', $accessNames);
								}
								break;
							case 'product_parent_code':
								if (!array_key_exists($record->product_parent_id, $this->parentIds))
								{
									$query = $this->db->getQuery(true)
										->select($this->db->quoteName('product_code'))
										->from($this->db->quoteName('#__hikashop_product'))
										->where($this->db->quoteName('product_id') . ' = ' . (int) $record->product_parent_id);
									$this->db->setQuery($query);

								$this->parentIds[$record->product_parent_id] = $this->db->loadResult();
								}

								$fieldValue = $this->parentIds[$record->product_parent_id];
								break;
							case 'product_created':
							case 'product_sale_start':
							case 'product_sale_end':
							case 'product_modified':
							case 'product_last_seen_date':
								$fieldValue = $this->fields->getDateFormat($fieldName, $record->$fieldName, $field->column_header);
								break;
							default:
								// See if we need to retrieve a custom field
								if (array_key_exists($fieldName, $this->characteristicFields))
								{
									$fieldValue = $this->getCharacteristicValue($record->product_id, $fieldName);
								}
								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldValue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}

	/**
	 * Convert prices to the new currency.
	 *
	 * @param   float   $product_price     The price to convert
	 * @param   string  $product_currency  The currency to convert to
	 *
	 * @return  float  A converted price.
	 *
	 * @since   6.0
	 *
	 * @throws  \RuntimeException
	 */
	private function convertPrice($product_price, $product_currency)
	{
		if (empty($product_price))
		{
			return $product_price;
		}
		else
		{
			// See if we need to convert the price
			if ($this->template->get('targetcurrency', '') !== '')
			{
				$query = $this->db->getQuery(true);
				$query->select($this->db->quoteName('currency_code') . ', ' . $this->db->quoteName('currency_rate'));
				$query->from($this->db->quoteName('#__csvi_currency'));
				$query->where(
					$this->db->quoteName('currency_code')
					. ' IN ('
						. $this->db->quote($product_currency) . ', ' . $this->db->quote($this->template->get('targetcurrency', 'EUR'))
					. ')'
				);
				$this->db->setQuery($query);
				$rates = $this->db->loadObjectList('currency_code');

				// Convert to base price
				$baseprice = $product_price / $rates[strtoupper($product_currency)]->currency_rate;

				// Convert to destination currency
				return $baseprice * $rates[strtoupper($this->template->get('targetcurrency', 'EUR'))]->currency_rate;
			}
			else
			{
				return $product_price;
			}
		}
	}

	/**
	 * Get a list of custom fields that can be used as available field.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 *
	 * @throws  \RuntimeException
	 */
	private function loadCustomFields()
	{
		$query = $this->db->getQuery(true)
			->select('TRIM(' . $this->db->quoteName('field_namekey') . ') AS ' . $this->db->quoteName('title'))
			->from($this->db->quoteName('#__hikashop_field'))
			->where($this->db->quoteName('field_table') . ' = ' . $this->db->quote('product'));
		$this->db->setQuery($query);
		$result = $this->db->loadColumn();

		if (!is_array($result))
		{
			$result = array();
		}

		$this->customFields = $result;
	}

	/**
	 * Load characteristics fields.
	 *
	 * @return  void.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \RuntimeException
	 */
	private function loadCharacteristicFields()
	{
		$query = $this->db->getQuery(true)
			->select('TRIM(' . $this->db->quoteName('characteristic_alias') . ') AS ' . $this->db->quoteName('title'))
			->select($this->db->quoteName('characteristic_id'))
			->from($this->db->quoteName('#__hikashop_characteristic'))
			->where($this->db->quoteName('characteristic_parent_id') . ' = 0');
		$this->db->setQuery($query);
		$result = $this->db->loadAssocList('title');

		if (!is_array($result))
		{
			$result = array();
		}

		$this->characteristicFields = $result;
	}

	/**
	 * Retrieve a list of product codes based on relation type.
	 *
	 * @param   int     $product_id  The ID of the product to get relations for.
	 * @param   string  $relation    The type of relation to search for.
	 *
	 * @return  object  List of product codes.
	 *
	 * @since   6.0
	 *
	 * @throws  \RuntimeException
	 */
	private function getProductRelations($product_id, $relation)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('#__hikashop_product.product_code'))
			->from($this->db->quoteName('#__hikashop_product_related'))
			->leftJoin(
				$this->db->quoteName('#__hikashop_product')
				. ' ON ' . $this->db->quoteName('#__hikashop_product.product_id')
				. ' = ' . $this->db->quoteName('#__hikashop_product_related.product_related_id')
			)
			->where($this->db->quoteName('#__hikashop_product_related.product_related_type') . ' = ' . $this->db->quote($relation))
			->where(
				$this->db->quoteName('#__hikashop_product_related.product_id')
				. ' = ' . (int) $product_id
			)
			->group($this->db->quoteName('#__hikashop_product.product_code'));
		$this->db->setQuery($query);

		return $this->db->loadColumn();
	}

	/**
	 * Retrieve the characteristics for a product.
	 *
	 * @param   int  $product_id  The ID of the product to get characteristics for.
	 *
	 * @return  object  List of characteristics.
	 *
	 * @since   6.0
	 *
	 * @throws  \RuntimeException
	 */
	private function getCharacteristics($product_id)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('characteristic_value'))
			->from($this->db->quoteName('#__hikashop_variant'))
			->leftJoin(
				$this->db->quoteName('#__hikashop_characteristic')
				. ' ON ' . $this->db->quoteName('#__hikashop_characteristic.characteristic_id')
				. ' = ' . $this->db->quoteName('#__hikashop_variant.variant_characteristic_id')
			)
			->where($this->db->quoteName('#__hikashop_variant.variant_product_id') . ' = ' . (int) $product_id)
			->where($this->db->quoteName('#__hikashop_variant.ordering') . ' > 0 ')
			->group($this->db->quoteName('characteristic_value'));
		$this->db->setQuery($query);

		return $this->db->loadColumn();
	}

	/**
	 * Retrieve the characteristic value.
	 *
	 * @param   int     $product_id  The ID of the product to get characteristics for.
	 * @param   string  $alias       The alias of the characteristic.
	 *
	 * @return  string  The characteristic value.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \RuntimeException
	 */
	private function getCharacteristicValue($product_id, $alias)
	{
		$value = '';

		// Get the parent ID
		if (array_key_exists($alias, $this->characteristicFields))
		{
			$parentId = $this->characteristicFields[$alias]['characteristic_id'];

			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('characteristic_value'))
				->from($this->db->quoteName('#__hikashop_variant', 'v'))
				->leftJoin(
					$this->db->quoteName('#__hikashop_characteristic', 'c')
					. ' ON ' . $this->db->quoteName('c.characteristic_id')
					. ' = ' . $this->db->quoteName('v.variant_characteristic_id')
				)
				->where($this->db->quoteName('v.variant_product_id') . ' = ' . (int) $product_id)
				->where($this->db->quoteName('c.characteristic_parent_id') . ' = ' . (int) $parentId);
			$this->db->setQuery($query);

			$value = $this->db->loadResult();
		}

		return $value;
	}
}
